import { __decorate } from "tslib";
import { attr, DOM, emptyArray, FASTElement } from "@microsoft/fast-element";
import { keyCodeEnter } from "@microsoft/fast-web-utilities";
export const supportsElementInternals = "ElementInternals" in window;
/**
 * Disable member ordering to keep property callbacks
 * grouped with property declaration
 */
export class FormAssociated extends FASTElement {
    constructor() {
        super();
        /**
         * The value of the element to be associated with the form
         */
        this.value = "";
        this.disabled = false;
        /**
         * Require the field prior to form submission
         */
        this.required = false;
        /**
         * These are events that are still fired by the proxy
         * element based on user / programmatic interaction.
         *
         * The proxy implementation should be transparent to
         * the app author, so block these events from emitting.
         */
        this.proxyEventsToBlock = ["change", "click"];
        if (supportsElementInternals) {
            this.elementInternals = this.attachInternals();
        }
    }
    /**
     * Must evaluate to true to enable elementInternals.
     * Feature detects API support and resolve respectively
     */
    static get formAssociated() {
        return supportsElementInternals;
    }
    /**
     * Returns the validty state of the element
     */
    get validity() {
        return supportsElementInternals
            ? this.elementInternals.validity
            : this.proxy.validity;
    }
    /**
     * Retrieve a reference to the associated form.
     * Returns null if not associated to any form.
     */
    get form() {
        return supportsElementInternals ? this.elementInternals.form : this.proxy.form;
    }
    /**
     * Retrieve the localized validation message,
     * or custom validation message if set.
     */
    get validationMessage() {
        return supportsElementInternals
            ? this.elementInternals.validationMessage
            : this.proxy.validationMessage;
    }
    /**
     * Whether the element will be validated when the
     * form is submitted
     */
    get willValidate() {
        return supportsElementInternals
            ? this.elementInternals.willValidate
            : this.proxy.willValidate;
    }
    /**
     * A reference to all associated label elements
     */
    get labels() {
        if (supportsElementInternals) {
            return Object.freeze(Array.from(this.elementInternals.labels));
        }
        else if (this.proxy instanceof HTMLElement &&
            this.proxy.ownerDocument &&
            this.id) {
            // Labels associated by wraping the element: <label><custom-element></custom-element></label>
            const parentLabels = this.proxy.labels;
            // Labels associated using the `for` attribute
            const forLabels = Array.from(this.proxy.getRootNode().querySelectorAll(`[for='${this.id}']`));
            const labels = parentLabels
                ? forLabels.concat(Array.from(parentLabels))
                : forLabels;
            return Object.freeze(labels);
        }
        else {
            return emptyArray;
        }
    }
    disabledChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.disabled = this.disabled;
        }
        DOM.queueUpdate(() => this.classList.toggle("disabled", this.disabled));
    }
    nameChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.name = this.name;
        }
    }
    requiredChanged() {
        if (this.proxy instanceof HTMLElement) {
            this.proxy.required = this.required;
        }
        DOM.queueUpdate(() => this.classList.toggle("required", this.required));
    }
    connectedCallback() {
        super.connectedCallback();
        if (!supportsElementInternals) {
            this.proxy.style.display = "none";
            this.appendChild(this.proxy);
            this.proxyEventsToBlock.forEach(name => this.proxy.addEventListener(name, this.stopPropagation));
            // These are typically mapped to the proxy during
            // property change callbacks, but during initialization
            // on the intial call of the callback, the proxy is
            // still undefined. We should find a better way to address this.
            this.proxy.disabled = this.disabled;
            this.proxy.required = this.required;
            if (typeof this.name === "string") {
                this.proxy.name = this.name;
            }
            if (typeof this.value === "string") {
                this.proxy.value = this.value;
            }
        }
    }
    disconnectedCallback() {
        this.proxyEventsToBlock.forEach(name => this.proxy.removeEventListener(name, this.stopPropagation));
    }
    /**
     * Return the current validity of the element
     */
    checkValidity() {
        return supportsElementInternals
            ? this.elementInternals.checkValidity()
            : this.proxy.checkValidity();
    }
    /**
     * Return the current validity of the element.
     * If false, fires an invalid event at the element.
     */
    reportValidity() {
        return supportsElementInternals
            ? this.elementInternals.reportValidity()
            : this.proxy.reportValidity();
    }
    /**
     * Set the validity of the control. In cases when the elementInternals object is not
     * available (and the proxy element is used to report validity), this function will
     * do nothing unless a message is provided, at which point the setCustomValidity method
     * of the proxy element will be invoked with the provided message.
     * @param flags - Validity flags
     * @param message - Optional message to supply
     * @param anchor - Optional element used by UA to display an interactive validation UI
     */
    setValidity(flags, message, anchor) {
        if (supportsElementInternals) {
            this.elementInternals.setValidity(flags, message, anchor);
        }
        else if (typeof message === "string") {
            this.proxy.setCustomValidity(message);
        }
    }
    /**
     * Invoked when a connected component's form or fieldset has it's disabled
     * state changed.
     * @param disabled - the disabled value of the form / fieldset
     */
    formDisabledCallback(disabled) {
        this.disabled = disabled;
    }
    /**
     *
     * @param value - The value to set
     * @param state - The state object provided to during session restores and when autofilling.
     */
    setFormValue(value, state) {
        if (supportsElementInternals) {
            this.elementInternals.setFormValue(value, state);
        }
    }
    keypressHandler(e) {
        switch (e.keyCode) {
            case keyCodeEnter:
                if (this.form instanceof HTMLFormElement) {
                    // Match native behavior
                    this.form.submit();
                }
                break;
        }
    }
    /**
     * Used to stop propagation of proxy element events
     * @param e - Event object
     */
    stopPropagation(e) {
        e.stopPropagation();
    }
}
__decorate([
    attr
], FormAssociated.prototype, "value", void 0);
__decorate([
    attr({ mode: "boolean" })
], FormAssociated.prototype, "disabled", void 0);
__decorate([
    attr
], FormAssociated.prototype, "name", void 0);
__decorate([
    attr({ mode: "boolean" })
], FormAssociated.prototype, "required", void 0);
